from .base_controller import BaseController
from models.media_download import MediaDownload, DownloadStatus
from models.media_counter import MediaCounter
from models.message import Message
from models.base import Session
from utils.api_service import APIService
from config import IMAGE_SAVE_PATH, VIDEOS_SAVE_PATH
import logging
import os

logger = logging.getLogger(__name__)

class MediaController(BaseController):

    @classmethod
    def _handle_media_download(cls, msg, message_type, timestamp):
        """Process media download synchronously"""
        session = Session()
        try:
            # Check for duplicate first
            media_info = msg.get('mediaInfo', {})
            file_hash = media_info.get('fileHash')

            # Only use file_hash for duplicate detection, not caption
            duplicate = Message.find_duplicate_message(
                message_type=message_type,
                file_hash=file_hash
            )

            if duplicate and duplicate.local_url:
                logger.info(f"Found duplicate media with hash {file_hash}, reusing file at {duplicate.local_url}")
                return duplicate.local_url, duplicate.file_size, duplicate.file_hash

            # No duplicate found, process download immediately
            return cls._process_media_download(msg, message_type, timestamp)

        finally:
            session.close()

    @classmethod
    def _process_media_download(cls, msg, message_type, timestamp):
        """Process the download synchronously"""
        session = Session()
        try:
            # Extract media info with fallbacks
            media_info = msg.get('mediaInfo', {})
            media_url = media_info.get('url')
            file_hash = media_info.get('fileHash')
            file_size = media_info.get('fileSize')
            message_id = msg.get('messageId')

            if not media_url or not message_id:
                logger.warning(f"Missing media URL or message ID: {message_id}")
                return None, None, None

            # Record download attempt
            MediaDownload.record_download_attempt(
                message_id=message_id,
                media_type=message_type,
                status=DownloadStatus.IN_PROGRESS
            )

            # Determine save path based on message type
            base_dir = IMAGE_SAVE_PATH if message_type == "image" else VIDEOS_SAVE_PATH
            extension = os.path.splitext(media_url)[1] or ('.jpg' if message_type == "image" else '.mp4')
            filename = f"{message_id}{extension}"
            save_path = os.path.join(base_dir, filename)

            if not os.path.exists(save_path):
                logger.info(f"Starting download for message {message_id}")
                success = APIService.download_media(media_url, save_path)

                if success:
                    MediaDownload.record_download_attempt(
                        message_id=message_id,
                        media_type=message_type,
                        status=DownloadStatus.COMPLETED,
                        local_url=save_path
                    )
                    MediaCounter.increment_counter(message_type)
                    return save_path, file_size, file_hash
                else:
                    MediaDownload.record_download_attempt(
                        message_id=message_id,
                        media_type=message_type,
                        status=DownloadStatus.FAILED,
                        error_message="Download failed"
                    )
                    return None, None, None

            return save_path, file_size, file_hash

        except Exception as e:
            logger.error(f"Error in media download: {str(e)}", exc_info=True)
            return None, None, None
        finally:
            session.close()

